@@ -13,12 +13,13 @@
 #
 # Copyright Buildbot Team Members
 
+import copy
 import datetime
 import json
 
 from twisted.internet import defer
 from twisted.internet import reactor
-from twisted.internet.protocol import ProcessProtocol
+from twisted.internet import utils
 from twisted.python import log
 
 from buildbot import config
@@ -27,6 +28,36 @@ from buildbot.changes import base
 from buildbot.changes.filter import ChangeFilter
 from buildbot.util import bytes2unicode
 from buildbot.util import httpclientservice
+from buildbot.util.protocol import LineProcessProtocol
+
+
+def _canonicalize_event(event):
+    """
+    Return an event dictionary which is consistent between the gerrit
+    event stream and the gerrit event log formats.
+    """
+    # For "patchset-created" the events-log JSON looks like:
+    #   "project": {"name": "buildbot"}
+    # while the stream-events JSON looks like:
+    #   "project": "buildbot"
+    # so we canonicalize them to the latter
+    if "change" not in event:
+        return event
+
+    change = event["change"]
+    if "project" not in change:
+        return event
+
+    project = change["project"]
+    if not isinstance(project, dict):
+        return event
+
+    if "name" not in project:
+        return event
+
+    event = copy.deepcopy(event)
+    event["change"]["project"] = project["name"]
+    return event
 
 
 class GerritChangeFilter(ChangeFilter):
@@ -74,7 +105,8 @@ class GerritChangeSourceBase(base.ChangeSource):
     def checkConfig(self,
                     gitBaseURL=None,
                     handled_events=("patchset-created", "ref-updated"),
-                    debug=False):
+                    debug=False,
+                    get_files=False):
 
         if gitBaseURL is None:
             config.error("gitBaseURL must be specified")
@@ -82,9 +114,11 @@ class GerritChangeSourceBase(base.ChangeSource):
     def reconfigService(self,
                         gitBaseURL=None,
                         handled_events=("patchset-created", "ref-updated"),
-                        debug=False):
+                        debug=False,
+                        get_files=False):
         self.gitBaseURL = gitBaseURL
         self.handled_events = list(handled_events)
+        self._get_files = get_files
         self.debug = debug
 
     def lineReceived(self, line):
@@ -123,23 +157,52 @@ class GerritChangeSourceBase(base.ChangeSource):
 
         properties = {}
         flatten(properties, "event", event)
-        event_with_change = "change" in event and "patchSet" in event
+        properties["event.source"] = self.__class__.__name__
         func_name = "eventReceived_%s" % event["type"].replace("-", "_")
         func = getattr(self, func_name, None)
-        if func is None and event_with_change:
+        if func is None:
             return self.addChangeFromEvent(properties, event)
-        elif func is None:
-            if self.debug:
-                log.msg("unsupported event %s" % (event["type"],))
-            return defer.succeed(None)
-        else:
-            return func(properties, event)
 
+        return func(properties, event)
+
+    @defer.inlineCallbacks
     def addChange(self, chdict):
-        d = self.master.data.updates.addChange(**chdict)
-        # eat failures..
-        d.addErrback(log.err, 'error adding change from GerritChangeSource')
-        return d
+        stampdict = {
+            "branch": chdict["branch"],
+            "revision": chdict["revision"],
+            "patch_author": chdict["author"],
+            "patch_comment": chdict["comments"],
+            "repository": chdict["repository"],
+            "project": chdict["project"],
+        }
+
+        stampid, found_existing = yield(
+             self.master.db.sourcestamps.findOrCreateId(**stampdict))
+
+        if found_existing:
+            if self.debug or True:
+                eventstr = "{}/{} -- {}:{}".format(
+                    self.gitBaseURL, chdict["project"], chdict["branch"],
+                    chdict["revision"])
+                message = (
+                    "gerrit: duplicate change event {} by {}"
+                    .format(eventstr, self.__class__.__name__))
+                log.msg(message.encode("utf-8"))
+            defer.returnValue(None)
+
+        if self.debug:
+            eventstr = "{} -- {}:{}".format(
+                chdict["repository"], chdict["branch"], chdict["revision"])
+            message = (
+                "gerrit: adding change from {} in {}"
+                .format(eventstr, self.__class__.__name__))
+            log.msg(message.encode("utf-8"))
+
+        try:
+            yield self.master.data.updates.addChange(**chdict)
+        except Exception:
+            # eat failures..
+            log.err('error adding change from GerritChangeSource')
 
     def getGroupingPolicyFromEvent(self, event):
         # At the moment, buildbot's change grouping strategy is hardcoded at various place
@@ -154,22 +217,40 @@ class GerritChangeSourceBase(base.ChangeSource):
                               event_change['number'])
         return event_change["branch"]
 
+    @defer.inlineCallbacks
     def addChangeFromEvent(self, properties, event):
+        if "change" not in event:
+            if self.debug:
+                log.msg("unsupported event %s" % (event["type"],))
+            return defer.returnValue(None)
 
-        if "change" in event and "patchSet" in event:
-            event_change = event["change"]
-            return self.addChange({
-                'author': _gerrit_user_to_author(event_change["owner"]),
-                'project': util.bytes2unicode(event_change["project"]),
-                'repository': "{}/{}".format(
-                    self.gitBaseURL, event_change["project"]),
-                'branch': self.getGroupingPolicyFromEvent(event),
-                'revision': event["patchSet"]["revision"],
-                'revlink': event_change["url"],
-                'comments': event_change["subject"],
-                'files': ["unknown"],
-                'category': event["type"],
-                'properties': properties})
+        if "patchSet" not in event:
+            if self.debug:
+                log.msg("unsupported event %s" % (event["type"],))
+            return defer.returnValue(None)
+
+        event = _canonicalize_event(event)
+        event_change = event["change"]
+
+        files = ["unknown"]
+        if self._get_files:
+            files = yield self.getFiles(
+                change=event_change["number"],
+                patchset=event["patchSet"]["number"]
+            )
+
+        yield self.addChange({
+            'author': _gerrit_user_to_author(event_change["owner"]),
+            'project': util.bytes2unicode(event_change["project"]),
+            'repository': "{}/{}".format(
+                self.gitBaseURL, event_change["project"]),
+            'branch': self.getGroupingPolicyFromEvent(event),
+            'revision': event["patchSet"]["revision"],
+            'revlink': event_change["url"],
+            'comments': event_change["subject"],
+            'files': files,
+            'category': event["type"],
+            'properties': properties})
 
     def eventReceived_ref_updated(self, properties, event):
         ref = event["refUpdate"]
@@ -243,36 +324,31 @@ class GerritChangeSource(GerritChangeSourceBase):
         self.streamProcessTimeout = self.STREAM_BACKOFF_MIN
         return super().reconfigService(**kwargs)
 
-    class LocalPP(ProcessProtocol):
+    class LocalPP(LineProcessProtocol):
 
         def __init__(self, change_source):
+            super().__init__()
             self.change_source = change_source
-            self.data = b""
 
         @defer.inlineCallbacks
-        def outReceived(self, data):
-            """Do line buffering."""
-            self.data += data
-            lines = self.data.split(b"\n")
-            # last line is either empty or incomplete
-            self.data = lines.pop(-1)
-            for line in lines:
-                if self.change_source.debug:
-                    log.msg(b"gerrit: " + line)
-                yield self.change_source.lineReceived(line)
-
-        def errReceived(self, data):
+        def outLineReceived(self, line):
             if self.change_source.debug:
-                log.msg(b"gerrit stderr: " + data)
+                log.msg(b"gerrit: " + line)
+            yield self.change_source.lineReceived(line)
 
-        def processEnded(self, status_object):
+        def errLineReceived(self, line):
+            if self.change_source.debug:
+                log.msg(b"gerrit stderr: " + line)
+
+        def processEnded(self, status):
+            super().processEnded(status)
             self.change_source.streamProcessStopped()
 
     def streamProcessStopped(self):
         self.process = None
 
         # if the service is stopped, don't try to restart the process
-        if not self.wantProcess or reactor._stopped:
+        if not self.wantProcess or not self.running:
             return
 
         now = util.now()
@@ -283,7 +359,7 @@ class GerritChangeSource(GerritChangeSourceBase):
             log.msg(
                 "'gerrit stream-events' failed; restarting after %ds"
                 % round(self.streamProcessTimeout))
-            reactor.callLater(
+            self.master.reactor.callLater(
                 self.streamProcessTimeout, self.startStreamProcess)
             self.streamProcessTimeout *= self.STREAM_BACKOFF_EXPONENT
             if self.streamProcessTimeout > self.STREAM_BACKOFF_MAX:
@@ -298,17 +374,49 @@ class GerritChangeSource(GerritChangeSourceBase):
             self.startStreamProcess()
             self.streamProcessTimeout = self.STREAM_BACKOFF_MIN
 
+    def _buildGerritCommand(self, *gerrit_args):
+        '''Get an ssh command list which invokes gerrit with the given args on the
+        remote host'''
+
+        cmd = [
+            "ssh",
+            "%s@%s" % (self.username, self.gerritserver),
+            "-p", str(self.gerritport)
+        ]
+
+        if self.identity_file is not None:
+            cmd.extend(["-i", self.identity_file])
+
+        cmd.append("gerrit")
+        cmd.extend(gerrit_args)
+        return cmd
+
     def startStreamProcess(self):
         if self.debug:
             log.msg("starting 'gerrit stream-events'")
+
+        cmd = self._buildGerritCommand("stream-events")
         self.lastStreamProcessStart = util.now()
-        uri = "%s@%s" % (self.username, self.gerritserver)
-        args = [uri, "-p", str(self.gerritport)]
-        if self.identity_file is not None:
-            args = args + ['-i', self.identity_file]
-        self.process = reactor.spawnProcess(
-            self.LocalPP(self), "ssh",
-            ["ssh"] + args + ["gerrit", "stream-events"], env=None)
+        self.process = reactor.spawnProcess(self.LocalPP(self), "ssh", cmd, env=None)
+
+    @defer.inlineCallbacks
+    def getFiles(self, change, patchset):
+        cmd = self._buildGerritCommand("query", str(change), "--format", "JSON",
+                                       "--files", "--patch-sets")
+
+        if self.debug:
+            log.msg("querying gerrit for changed files in change %s/%s: %s" %
+                    (change, patchset, cmd))
+
+        out = yield utils.getProcessOutput(cmd[0], cmd[1:], env=None)
+        out = out.splitlines()[0]
+        res = json.loads(bytes2unicode(out))
+
+        if res.get("rowCount") == 0:
+            return ["unknown"]
+
+        patchsets = {i["number"]: i["files"] for i in res["patchSets"]}
+        return [i["file"] for i in patchsets[int(patchset)]]
 
     def activate(self):
         self.wantProcess = True
@@ -333,12 +441,14 @@ class GerritChangeSource(GerritChangeSourceBase):
 class GerritEventLogPoller(GerritChangeSourceBase):
 
     POLL_INTERVAL_SEC = 30
+    FIRST_FETCH_LOOKBACK_DAYS = 30
 
     def checkConfig(self,
                     baseURL,
                     auth,
                     pollInterval=POLL_INTERVAL_SEC,
                     pollAtLaunch=True,
+                    firstFetchLookback=FIRST_FETCH_LOOKBACK_DAYS,
                     **kwargs):
         if self.name is None:
             self.name = "GerritEventLogPoller:{}".format(baseURL)
@@ -350,6 +460,7 @@ class GerritEventLogPoller(GerritChangeSourceBase):
                         auth,
                         pollInterval=POLL_INTERVAL_SEC,
                         pollAtLaunch=True,
+                        firstFetchLookback=FIRST_FETCH_LOOKBACK_DAYS,
                         **kwargs):
 
         yield super().reconfigService(**kwargs)
@@ -362,6 +473,9 @@ class GerritEventLogPoller(GerritChangeSourceBase):
         self._http = yield httpclientservice.HTTPClientService.getService(
             self.master, baseURL, auth=auth)
 
+        self._first_fetch_lookback = firstFetchLookback
+        self._last_event_time = None
+
     @staticmethod
     def now():
         """patchable now (datetime is not patchable as builtin)"""
@@ -371,10 +485,16 @@ class GerritEventLogPoller(GerritChangeSourceBase):
     def poll(self):
         last_event_ts = yield self.master.db.state.getState(self._oid, 'last_event_ts', None)
         if last_event_ts is None:
-            last_event = self.now()
+            # If there is not last event time stored in the database, then set
+            # the last event time to some historical look-back
+            last_event = self.now() - datetime.timedelta(days=self._first_fetch_lookback)
         else:
             last_event = datetime.datetime.utcfromtimestamp(last_event_ts)
-        last_event_formatted = last_event.strftime("%Y-%d-%m %H:%M:%S")
+        last_event_formatted = last_event.strftime("%Y-%m-%d %H:%M:%S")
+
+        if self.debug:
+            log.msg("Polling gerrit: {}".format(last_event_formatted).encode("utf-8"))
+
         res = yield self._http.get("/plugins/events-log/events/", params=dict(t1=last_event_formatted))
         lines = yield res.content()
         for line in lines.splitlines():
@@ -387,6 +507,14 @@ class GerritEventLogPoller(GerritChangeSourceBase):
             yield self.master.db.state.setState(self._oid, 'last_event_ts', event['eventCreatedOn'])
         return res
 
+    @defer.inlineCallbacks
+    def getFiles(self, change, patchset):
+        res = yield self._http.get("/changes/%s/revisions/%s/files/" % (change, patchset))
+        res = yield res.content()
+
+        res = res.splitlines()[1].decode('utf8')  # the first line of every response is `)]}'`
+        return list(json.loads(res))
+
     # FIXME this copy the code from PollingChangeSource
     # but as PollingChangeSource and its subclasses need to be ported to reconfigurability
     # we can't use it right now
