@@ -1,4 +1,5 @@
-# Runs the build-bot as a Windows service.
+# pylint: disable=import-outside-toplevel
+# # Runs the build-bot as a Windows service.
 # To use:
 # * Install and configure buildbot as per normal (ie, running
 #  'setup.py install' from the source directory).
@@ -245,27 +246,26 @@ class BBService(win32serviceutil.ServiceFramework):
             if rc == win32event.WAIT_OBJECT_0:
                 # user sent a stop service request
                 break
-            else:
-                # A child process died.  For now, just log the output
-                # and forget the process.
-                index = rc - win32event.WAIT_OBJECT_0 - 1
-                bbdir, dead_handle, dead_thread, output_blocks = \
-                    child_infos[index]
-                status = win32process.GetExitCodeProcess(dead_handle)
-                output = "".join(output_blocks)
-                if not output:
-                    output = "The child process generated no output. " \
-                             "Please check the twistd.log file in the " \
-                             "indicated directory."
+            # A child process died.  For now, just log the output
+            # and forget the process.
+            index = rc - win32event.WAIT_OBJECT_0 - 1
+            bbdir, dead_handle, _, output_blocks = \
+                child_infos[index]
+            status = win32process.GetExitCodeProcess(dead_handle)
+            output = "".join(output_blocks)
+            if not output:
+                output = ("The child process generated no output. "
+                          "Please check the twistd.log file in the "
+                          "indicated directory.")
 
-                self.warning("BuildBot for directory %r terminated with "
-                             "exit code %d.\n%s" % (bbdir, status, output))
+            self.warning("BuildBot for directory %r terminated with "
+                         "exit code %d.\n%s" % (bbdir, status, output))
 
-                del child_infos[index]
+            del child_infos[index]
 
-                if not child_infos:
-                    self.warning("All BuildBot child processes have "
-                                 "terminated.  Service stopping.")
+            if not child_infos:
+                self.warning("All BuildBot child processes have "
+                             "terminated.  Service stopping.")
 
         # Either no child processes left, or stop event set.
         self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
@@ -469,7 +469,7 @@ def RegisterWithFirewall(exe_name, description):
 
 
 def CustomInstall(opts):
-    # Register this process with the Windows Firewaall
+    # Register this process with the Windows Firewall
     import pythoncom
     try:
         RegisterWithFirewall(sys.executable, "BuildBot")
