@@ -16,15 +16,16 @@
 import mock
 
 from twisted.internet import defer
-from twisted.internet import reactor
-from twisted.internet import task
 from twisted.trial import unittest
 
 from buildbot.changes import base
 from buildbot.test.util import changesource
+from buildbot.test.util.misc import TestReactorMixin
 
 
-class TestChangeSource(changesource.ChangeSourceMixin, unittest.TestCase):
+class TestChangeSource(changesource.ChangeSourceMixin,
+                       TestReactorMixin,
+                       unittest.TestCase):
     timeout = 120
 
     class Subclass(base.ChangeSource):
@@ -32,6 +33,7 @@ class TestChangeSource(changesource.ChangeSourceMixin, unittest.TestCase):
 
     @defer.inlineCallbacks
     def setUp(self):
+        self.setUpTestReactor()
         yield self.setUpChangeSource()
 
     def tearDown(self):
@@ -73,7 +75,9 @@ class TestChangeSource(changesource.ChangeSourceMixin, unittest.TestCase):
         self.assertFalse(cs.active)
 
 
-class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase):
+class TestPollingChangeSource(changesource.ChangeSourceMixin,
+                              TestReactorMixin,
+                              unittest.TestCase):
     timeout = 120
 
     class Subclass(base.PollingChangeSource):
@@ -81,11 +85,7 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
 
     @defer.inlineCallbacks
     def setUp(self):
-        # patch in a Clock so we can manipulate the reactor's time
-        self.clock = task.Clock()
-        self.patch(reactor, 'callLater', self.clock.callLater)
-        self.patch(reactor, 'seconds', self.clock.seconds)
-
+        self.setUpTestReactor()
         yield self.setUpChangeSource()
 
         self.attachChangeSource(self.Subclass(name="DummyCS"))
@@ -93,14 +93,15 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
     def tearDown(self):
         return self.tearDownChangeSource()
 
+    @defer.inlineCallbacks
     def runClockFor(self, _, secs):
-        self.clock.pump([1.0] * secs)
+        yield self.reactor.pump([1.0] * secs)
 
     def test_loop_loops(self):
         # track when poll() gets called
         loops = []
         self.changesource.poll = \
-            lambda: loops.append(self.clock.seconds())
+            lambda: loops.append(self.reactor.seconds())
 
         self.changesource.pollInterval = 5
         self.startChangeSource()
@@ -111,7 +112,7 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
         def check(_):
             # note that it does *not* poll at time 0
             self.assertEqual(loops, [5.0, 10.0])
-        reactor.callWhenRunning(d.callback, None)
+        self.reactor.callWhenRunning(d.callback, None)
         return d
 
     def test_loop_exception(self):
@@ -119,7 +120,7 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
         loops = []
 
         def poll():
-            loops.append(self.clock.seconds())
+            loops.append(self.reactor.seconds())
             raise RuntimeError("oh noes")
         self.changesource.poll = poll
 
@@ -134,7 +135,7 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
             # note that it keeps looping after error
             self.assertEqual(loops, [5.0, 10.0])
             self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 2)
-        reactor.callWhenRunning(d.callback, None)
+        self.reactor.callWhenRunning(d.callback, None)
         return d
 
     def test_poll_only_if_activated(self):
@@ -144,7 +145,7 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
 
         loops = []
         self.changesource.poll = \
-            lambda: loops.append(self.clock.seconds())
+            lambda: loops.append(self.reactor.seconds())
 
         self.changesource.pollInterval = 5
         self.startChangeSource()
@@ -157,14 +158,14 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
             # it doesn't do anything because it was already claimed
             self.assertEqual(loops, [])
 
-        reactor.callWhenRunning(d.callback, None)
+        self.reactor.callWhenRunning(d.callback, None)
         return d
 
     def test_pollAtLaunch(self):
         # track when poll() gets called
         loops = []
         self.changesource.poll = \
-            lambda: loops.append(self.clock.seconds())
+            lambda: loops.append(self.reactor.seconds())
 
         self.changesource.pollInterval = 5
         self.changesource.pollAtLaunch = True
@@ -177,21 +178,20 @@ class TestPollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase)
         def check(_):
             # note that it *does* poll at time 0
             self.assertEqual(loops, [0.0, 5.0, 10.0])
-        reactor.callWhenRunning(d.callback, None)
+        self.reactor.callWhenRunning(d.callback, None)
         return d
 
 
-class TestReconfigurablePollingChangeSource(changesource.ChangeSourceMixin, unittest.TestCase):
+class TestReconfigurablePollingChangeSource(changesource.ChangeSourceMixin,
+                                            TestReactorMixin,
+                                            unittest.TestCase):
 
     class Subclass(base.ReconfigurablePollingChangeSource):
         pass
 
     @defer.inlineCallbacks
     def setUp(self):
-        # patch in a Clock so we can manipulate the reactor's time
-        self.clock = task.Clock()
-        self.patch(reactor, 'callLater', self.clock.callLater)
-        self.patch(reactor, 'seconds', self.clock.seconds)
+        self.setUpTestReactor()
 
         yield self.setUpChangeSource()
 
@@ -200,15 +200,16 @@ class TestReconfigurablePollingChangeSource(changesource.ChangeSourceMixin, unit
     def tearDown(self):
         return self.tearDownChangeSource()
 
+    @defer.inlineCallbacks
     def runClockFor(self, secs):
-        self.clock.pump([1.0] * secs)
+        yield self.reactor.pump([1.0] * secs)
 
     @defer.inlineCallbacks
     def test_loop_loops(self):
         # track when poll() gets called
         loops = []
         self.changesource.poll = \
-            lambda: loops.append(self.clock.seconds())
+            lambda: loops.append(self.reactor.seconds())
 
         yield self.startChangeSource()
         yield self.changesource.reconfigServiceWithSibling(self.Subclass(
@@ -224,7 +225,7 @@ class TestReconfigurablePollingChangeSource(changesource.ChangeSourceMixin, unit
         loops = []
 
         def poll():
-            loops.append(self.clock.seconds())
+            loops.append(self.reactor.seconds())
             raise RuntimeError("oh noes")
         self.changesource.poll = poll
 
@@ -245,7 +246,7 @@ class TestReconfigurablePollingChangeSource(changesource.ChangeSourceMixin, unit
 
         loops = []
         self.changesource.poll = \
-            lambda: loops.append(self.clock.seconds())
+            lambda: loops.append(self.reactor.seconds())
 
         yield self.startChangeSource()
         yield self.changesource.reconfigServiceWithSibling(self.Subclass(
@@ -261,7 +262,7 @@ class TestReconfigurablePollingChangeSource(changesource.ChangeSourceMixin, unit
         # track when poll() gets called
         loops = []
         self.changesource.poll = \
-            lambda: loops.append(self.clock.seconds())
+            lambda: loops.append(self.reactor.seconds())
         yield self.startChangeSource()
         yield self.changesource.reconfigServiceWithSibling(self.Subclass(
             name="DummyCS", pollInterval=5, pollAtLaunch=True))
