@@ -13,8 +13,6 @@
 #
 # Copyright Buildbot Team Members
 
-from future.utils import text_type
-
 import random
 
 import mock
@@ -23,13 +21,13 @@ from twisted.internet import defer
 from twisted.trial import unittest
 
 from buildbot import config
-from buildbot import locks
 from buildbot.process import builder
 from buildbot.process import factory
 from buildbot.process.properties import Properties
 from buildbot.process.properties import renderer
 from buildbot.test.fake import fakedb
 from buildbot.test.fake import fakemaster
+from buildbot.test.util.misc import TestReactorMixin
 from buildbot.test.util.warnings import assertProducesWarning
 from buildbot.util import epoch2datetime
 from buildbot.worker import AbstractLatentWorker
@@ -39,7 +37,7 @@ class BuilderMixin:
 
     def setUpBuilderMixin(self):
         self.factory = factory.BuildFactory()
-        self.master = fakemaster.make_master(testcase=self, wantData=True)
+        self.master = fakemaster.make_master(self, wantData=True)
         self.mq = self.master.mq
         self.db = self.master.db
 
@@ -86,9 +84,10 @@ class FakeWorker:
         self.workername = workername
 
 
-class TestBuilder(BuilderMixin, unittest.TestCase):
+class TestBuilder(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     def setUp(self):
+        self.setUpTestReactor()
         # a collection of rows that would otherwise clutter up every test
         self.setUpBuilderMixin()
         self.base_rows = [
@@ -279,8 +278,7 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
     def test_canStartBuild_cant_acquire_locks_but_no_locks(self):
         yield self.makeBuilder()
 
-        self.bldr.botmaster.getLockFromLockAccess = \
-            mock.Mock(return_value='dummy')
+        self.bldr.botmaster.getLockFromLockAccesses = mock.Mock(return_value=[])
 
         wfb = mock.Mock()
         wfb.worker = FakeWorker('worker')
@@ -295,12 +293,7 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
     def test_canStartBuild_with_locks(self):
         yield self.makeBuilder()
 
-        self.bldr.botmaster.getLockFromLockAccess = \
-            mock.Mock(return_value='dummy')
-
-        lock1 = mock.Mock(spec=locks.MasterLock)
-        lock1.name = "masterlock"
-        self.bldr.config.locks = [lock1]
+        self.bldr.botmaster.getLockFromLockAccesses = mock.Mock(return_value=[mock.Mock()])
 
         wfb = mock.Mock()
         wfb.worker = FakeWorker('worker')
@@ -315,26 +308,14 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
     def test_canStartBuild_with_renderable_locks(self):
         yield self.makeBuilder()
 
-        self.bldr.botmaster.getLockFromLockAccess = \
-            mock.Mock(return_value='dummy')
-
-        self.bldr.botmaster.getLockFromLockAccess = \
-            mock.Mock(return_value='dummy')
-
-        lock1 = mock.Mock(spec=locks.MasterLock)
-        lock1.name = "masterlock"
-
-        lock2 = mock.Mock(spec=locks.WorkerLock)
-        lock2.name = "workerlock"
+        self.bldr.botmaster.getLockFromLockAccesses = mock.Mock(return_value=[mock.Mock()])
 
         renderedLocks = [False]
 
         @renderer
         def rendered_locks(props):
             renderedLocks[0] = True
-            access1 = locks.LockAccess(lock1, 'counting')
-            access2 = locks.LockAccess(lock2, 'exclusive')
-            return [access1, access2]
+            return [mock.Mock()]
 
         self.bldr.config.locks = rendered_locks
 
@@ -417,7 +398,7 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
     @defer.inlineCallbacks
     def test_getBuilderId(self):
         self.factory = factory.BuildFactory()
-        self.master = fakemaster.make_master(testcase=self, wantData=True)
+        self.master = fakemaster.make_master(self, wantData=True)
         # only include the necessary required config, plus user-requested
         self.bldr = builder.Builder('bldr')
         self.bldr.master = self.master
@@ -433,8 +414,9 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
         self.assertEqual(builderid, 13)
         fbi.assert_not_called()
 
+    @defer.inlineCallbacks
     def test_expectations_deprecated(self):
-        self.successResultOf(self.makeBuilder())
+        yield self.makeBuilder()
 
         with assertProducesWarning(
                 Warning,
@@ -457,9 +439,10 @@ class TestBuilder(BuilderMixin, unittest.TestCase):
         self.assertEquals(props.getProperty('cuckoo'), 42)
 
 
-class TestGetBuilderId(BuilderMixin, unittest.TestCase):
+class TestGetBuilderId(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
     @defer.inlineCallbacks
@@ -476,13 +459,15 @@ class TestGetBuilderId(BuilderMixin, unittest.TestCase):
         fbi.assert_called_once_with('b1')
         # check that the name was unicodified
         arg = fbi.mock_calls[0][1][0]
-        self.assertIsInstance(arg, text_type)
+        self.assertIsInstance(arg, str)
 
 
-class TestGetOldestRequestTime(BuilderMixin, unittest.TestCase):
+class TestGetOldestRequestTime(TestReactorMixin, BuilderMixin,
+                               unittest.TestCase):
 
     @defer.inlineCallbacks
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
         # a collection of rows that would otherwise clutter up every test
@@ -531,10 +516,11 @@ class TestGetOldestRequestTime(BuilderMixin, unittest.TestCase):
         self.assertEqual(rqtime, None)
 
 
-class TestGetNewestCompleteTime(BuilderMixin, unittest.TestCase):
+class TestGetNewestCompleteTime(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     @defer.inlineCallbacks
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
         # a collection of rows that would otherwise clutter up every test
@@ -571,11 +557,12 @@ class TestGetNewestCompleteTime(BuilderMixin, unittest.TestCase):
         self.assertEqual(rqtime, None)
 
 
-class TestReconfig(BuilderMixin, unittest.TestCase):
+class TestReconfig(TestReactorMixin, BuilderMixin, unittest.TestCase):
 
     """Tests that a reconfig properly updates all attributes"""
 
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpBuilderMixin()
 
     @defer.inlineCallbacks
