@@ -18,8 +18,10 @@ import re
 import mock
 
 from twisted.internet import defer
+from twisted.internet import reactor
 from twisted.trial import unittest
 
+from buildbot.process.results import FAILURE
 from buildbot.process.results import SUCCESS
 from buildbot.reporters import words
 from buildbot.test.fake import fakedb
@@ -28,13 +30,24 @@ from buildbot.test.util.misc import TestReactorMixin
 from buildbot.util import datetime2epoch
 
 
-class TestContactChannel(TestReactorMixin, unittest.TestCase):
+class ContactMixin(TestReactorMixin):
+    botClass = words.StatusBot
+
+    channelClass = words.Channel
+    contactClass = words.Contact
+    USER = "me"
+    CHANNEL = "#buildbot"
 
     BUILDER_NAMES = ['builder1', 'builder2']
     BUILDER_IDS = [23, 45]
 
+    @defer.inlineCallbacks
     def setUp(self):
         self.setUpTestReactor()
+        self.patch(reactor, 'callLater', self.reactor.callLater)
+        self.patch(reactor, 'seconds', self.reactor.seconds)
+        self.patch(reactor, 'stop', self.reactor.stop)
+
         self.master = fakemaster.make_master(self, wantMq=True, wantData=True,
                                              wantDb=True)
 
@@ -42,30 +55,19 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
             self.master.db.builders.addTestBuilder(
                 builderid=builderid, name=name)
 
-        # I think the 'bot' part of this is actually going away ...
-        # TO REMOVE:
-
-        self.bot = mock.Mock(name='IRCStatusBot-instance')
+        self.bot = self.botClass(notify_events={'success': 1, 'failure': 1})
+        self.bot.channelClass = self.channelClass
+        self.bot.contactClass = self.contactClass
         self.bot.nickname = 'nick'
-        self.bot.notify_events = {'success': 1, 'failure': 1}
-        self.bot.useRevisions = False
-        self.bot.useColors = False
+        self.missing_workers = set()
 
         # fake out subscription/unsubscription
         self.subscribed = False
 
-        def subscribe(contact):
-            self.subscribed = True
-        self.bot.status.subscribe = subscribe
-
-        def unsubscribe(contact):
-            self.subscribed = False
-        self.bot.status.unsubscribe = unsubscribe
-
         # fake out clean shutdown
-        self.bot.master = self.master
+        self.bot.parent = self
         self.bot.master.botmaster = mock.Mock(
-            name='IRCStatusBot-instance.master.botmaster')
+            name='StatusBot-instance.master.botmaster')
         self.bot.master.botmaster.shuttingDown = False
 
         def cleanShutdown():
@@ -76,69 +78,88 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
             self.bot.master.botmaster.shuttingDown = False
         self.bot.master.botmaster.cancelCleanShutdown = cancelCleanShutdown
 
-        self.contact = words.Contact(self.bot, user='me', channel='#buildbot',
-                                     _reactor=self.reactor)
-        self.contact.setServiceParent(self.master)
-        return self.master.startService()
+        self.contact = self.contactClass(user=self.USER,
+                                         channel=self.bot.getChannel(self.CHANNEL))
+        yield self.contact.channel.setServiceParent(self.master)
+        yield self.master.startService()
 
     def patch_send(self):
         self.sent = []
 
         def send(msg):
-            self.sent.append(msg)
-        self.contact.send = send
-
-    def patch_act(self):
-        self.actions = []
-
-        def act(msg):
-            self.actions.append(msg)
-        self.contact.act = act
+            if not isinstance(msg, (list, tuple)):
+                msg = (msg,)
+            for m in msg:
+                self.sent.append(m)
+        self.contact.channel.send = send
 
     @defer.inlineCallbacks
-    def do_test_command(self, command, args='', clock_ticks=None,
-                        exp_usage=True, exp_UsageError=False, allowShutdown=False,
-                        shuttingDown=False):
-        cmd = getattr(self.contact, 'command_' + command.upper())
+    def do_test_command(self, command, args='', contact=None, clock_ticks=None,
+                        exp_usage=True, exp_UsageError=False,
+                        shuttingDown=False, **kwargs):
+        if contact is None:
+            contact = self.contact
+
+        cmd = getattr(contact, 'command_' + command.upper())
 
         if exp_usage:
             self.assertTrue(hasattr(cmd, 'usage'))
 
         self.patch_send()
-        self.patch_act()
-        self.bot.factory.allowShutdown = allowShutdown
         self.bot.master.botmaster.shuttingDown = shuttingDown
 
         if exp_UsageError:
             try:
-                yield cmd(args)
+                yield cmd(args, **kwargs)
             except words.UsageError:
                 return
             else:
                 self.fail("no UsageError")
         else:
-            cmd(args)
+            yield cmd(args, **kwargs)
         if clock_ticks:
             self.reactor.pump(clock_ticks)
 
-    # tests
+    def setupSomeBuilds(self):
+        self.master.db.insertTestData([
+            # Three builds on builder#0, One build on builder#1
+            fakedb.Build(id=13, masterid=88, workerid=13,
+                         builderid=self.BUILDER_IDS[0],
+                         buildrequestid=82, number=3),
+            fakedb.Build(id=14, masterid=88, workerid=13,
+                         builderid=self.BUILDER_IDS[0],
+                         buildrequestid=83, number=4),
+            fakedb.Build(id=15, masterid=88, workerid=13,
+                         builderid=self.BUILDER_IDS[1],
+                         buildrequestid=84, number=5),
+            fakedb.Build(id=16, masterid=88, workerid=13,
+                         builderid=self.BUILDER_IDS[0],
+                         buildrequestid=85, number=6),
+        ])
+        self.master.db.builds.finishBuild(buildid=14, results=SUCCESS)
 
-    def test_doSilly(self):
-        self.patch_send()
-        silly_prompt, silly_response = list(self.contact.silly.items())[0]
+    def setup_multi_builders(self):
+        # Make first builder configured, but not connected
+        # Make second builder configured and connected
+        self.master.db.insertTestData([
+            fakedb.Worker(id=1, name='linux1', info={}),  # connected one
+            fakedb.Worker(id=2, name='linux2', info={}),  # disconnected one
+            fakedb.BuilderMaster(
+                id=4012, masterid=13, builderid=self.BUILDER_IDS[0]),
+            fakedb.BuilderMaster(
+                id=4013, masterid=13, builderid=self.BUILDER_IDS[1]),
+            fakedb.ConfiguredWorker(id=14013,
+                                    workerid=2, buildermasterid=4012),
+            fakedb.ConfiguredWorker(id=14013,
+                                    workerid=1, buildermasterid=4013),
+        ])
 
-        self.contact.doSilly(silly_prompt)
-        self.reactor.pump([0.5] * 20)
 
-        self.assertEqual(self.sent, silly_response)
+class TestContact(ContactMixin, unittest.TestCase):
 
-    # TODO: remaining commands
-    # (all depend on status, which interface will change soon)
-
-    @defer.inlineCallbacks
-    def test_command_mute(self):
-        yield self.do_test_command('mute')
-        self.assertTrue(self.contact.muted)
+    def test_channel_service(self):
+        self.assertTrue(self.contact.channel.running)
+        self.contact.channel.stopService()
 
     @defer.inlineCallbacks
     def test_command_notify0(self):
@@ -146,30 +167,30 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         yield self.do_test_command('notify', args="invalid arg", exp_UsageError=True)
         yield self.do_test_command('notify', args="on")
         self.assertEqual(
-            self.sent, ["The following events are being notified: ['finished', 'started']"])
+            self.sent, ["The following events are being notified: finished, started."])
         yield self.do_test_command('notify', args="off")
         self.assertEqual(
-            self.sent, ['The following events are being notified: []'])
+            self.sent, ['No events are being notified.'])
         yield self.do_test_command('notify', args="on started")
         self.assertEqual(
-            self.sent, ["The following events are being notified: ['started']"])
+            self.sent, ["The following events are being notified: started."])
         yield self.do_test_command('notify', args="off started")
         self.assertEqual(
-            self.sent, ['The following events are being notified: []'])
+            self.sent, ['No events are being notified.'])
         yield self.assertFailure(
             self.do_test_command('notify', args="off finished"),
             KeyError)
         yield self.do_test_command('notify', args="list")
         self.assertEqual(
-            self.sent, ['The following events are being notified: []'])
+            self.sent, ['No events are being notified.'])
 
     @defer.inlineCallbacks
     def notify_build_test(self, notify_args):
         self.bot.tags = None
         yield self.test_command_watch_builder0()
         yield self.do_test_command('notify', args=notify_args)
-        buildStarted = self.contact.subscribed[0].callback
-        buildFinished = self.contact.subscribed[1].callback
+        buildStarted = self.contact.channel.subscribed[0].callback
+        buildFinished = self.contact.channel.subscribed[1].callback
         for buildid in (13, 14, 16):
             self.master.db.builds.finishBuild(buildid=buildid, results=SUCCESS)
             build = yield self.master.db.builds.getBuild(buildid)
@@ -182,25 +203,44 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     def test_command_notify_build_finished(self):
         self.notify_build_test("on finished")
 
+    def test_command_notify_build_better(self):
+        self.notify_build_test("on better")
+
+    def test_command_notify_build_worse(self):
+        self.notify_build_test("on worse")
+
+    def test_command_notify_build_problem(self):
+        self.notify_build_test("on problem")
+
+    def test_command_notify_build_recovery(self):
+        self.notify_build_test("on recovery")
+
     def test_command_notify_build_started_finished(self):
         self.notify_build_test("on")
 
     @defer.inlineCallbacks
-    def test_command_unmute(self):
-        self.contact.muted = True
-        yield self.do_test_command('unmute')
-        self.assertFalse(self.contact.muted)
+    def test_notify_missing_worker(self):
+        self.patch_send()
+        yield self.do_test_command('notify', args='on worker')
+        missing_worker = self.contact.channel.subscribed[2].callback
+        missing_worker((None, None, 'missing'), dict(workerid=1, name="work", last_connection="sometime"))
+        self.assertEquals(self.sent[1], "Worker `work` is missing. It was seen last on sometime.")
+        self.assertIn(1, self.contact.channel.missing_workers)
 
     @defer.inlineCallbacks
-    def test_command_unmute_not_muted(self):
-        yield self.do_test_command('unmute')
-        self.assertFalse(self.contact.muted)
-        self.assertIn("hadn't told me to be quiet", self.sent[0])
+    def test_notify_worker_is_back(self):
+        self.patch_send()
+        yield self.do_test_command('notify', args='on worker')
+        self.contact.channel.missing_workers.add(1)
+        missing_worker = self.contact.channel.subscribed[2].callback
+        missing_worker((None, None, 'connected'), dict(workerid=1, name="work", last_connection="sometime"))
+        self.assertEquals(self.sent[1], "Worker `work` is back online.")
+        self.assertNotIn(1, self.contact.channel.missing_workers)
 
     @defer.inlineCallbacks
     def test_command_help_noargs(self):
         yield self.do_test_command('help')
-        self.assertIn('help on what', self.sent[0])
+        self.assertIn('help - ', '\n'.join(self.sent))
 
     @defer.inlineCallbacks
     def test_command_help_arg(self):
@@ -278,45 +318,33 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     @defer.inlineCallbacks
     def test_command_shutdown(self):
         yield self.do_test_command('shutdown', exp_UsageError=True)
-        self.assertEqual(self.bot.factory.allowShutdown, False)
-        self.assertEqual(self.bot.master.botmaster.shuttingDown, False)
-
-    @defer.inlineCallbacks
-    def test_command_shutdown_dissalowed(self):
-        yield self.do_test_command('shutdown', args='check', exp_UsageError=True)
-        self.assertEqual(self.bot.factory.allowShutdown, False)
         self.assertEqual(self.bot.master.botmaster.shuttingDown, False)
 
     @defer.inlineCallbacks
     def test_command_shutdown_check_running(self):
-        yield self.do_test_command('shutdown', args='check', allowShutdown=True, shuttingDown=False)
-        self.assertEqual(self.bot.factory.allowShutdown, True)
+        yield self.do_test_command('shutdown', args='check', shuttingDown=False)
         self.assertEqual(self.bot.master.botmaster.shuttingDown, False)
         self.assertIn('buildbot is running', self.sent[0])
 
     @defer.inlineCallbacks
     def test_command_shutdown_check_shutting_down(self):
-        yield self.do_test_command('shutdown', args='check', allowShutdown=True, shuttingDown=True)
-        self.assertEqual(self.bot.factory.allowShutdown, True)
+        yield self.do_test_command('shutdown', args='check', shuttingDown=True)
         self.assertEqual(self.bot.master.botmaster.shuttingDown, True)
         self.assertIn('buildbot is shutting down', self.sent[0])
 
     @defer.inlineCallbacks
     def test_command_shutdown_start(self):
-        yield self.do_test_command('shutdown', args='start', allowShutdown=True, shuttingDown=False)
-        self.assertEqual(self.bot.factory.allowShutdown, True)
+        yield self.do_test_command('shutdown', args='start', shuttingDown=False)
         self.assertEqual(self.bot.master.botmaster.shuttingDown, True)
 
     @defer.inlineCallbacks
     def test_command_shutdown_stop(self):
-        yield self.do_test_command('shutdown', args='stop', allowShutdown=True, shuttingDown=True)
-        self.assertEqual(self.bot.factory.allowShutdown, True)
+        yield self.do_test_command('shutdown', args='stop', shuttingDown=True)
         self.assertEqual(self.bot.master.botmaster.shuttingDown, False)
 
     @defer.inlineCallbacks
     def test_command_shutdown_now(self):
-        yield self.do_test_command('shutdown', args='now', allowShutdown=True)
-        self.assertEqual(self.bot.factory.allowShutdown, True)
+        yield self.do_test_command('shutdown', args='now')
         self.assertEqual(self.bot.master.botmaster.shuttingDown, False)
         self.assertTrue(self.reactor.stop_called)
 
@@ -328,27 +356,10 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     @defer.inlineCallbacks
     def test_command_commands(self):
         yield self.do_test_command('commands')
-        self.assertIn('buildbot commands', self.sent[0])
-
-    @defer.inlineCallbacks
-    def test_command_destroy(self):
-        yield self.do_test_command('destroy', exp_usage=False)
-        self.assertEqual(self.actions, ['readies phasers'])
-
-    @defer.inlineCallbacks
-    def test_command_dance(self):
-        yield self.do_test_command('dance', clock_ticks=[1.0] * 10, exp_usage=False)
-        self.assertTrue(self.sent)  # doesn't matter what it sent
-
-    @defer.inlineCallbacks
-    def test_command_hustle(self):
-        yield self.do_test_command('hustle', clock_ticks=[1.0] * 2, exp_usage=False)
-        self.assertEqual(self.actions, ['does the hustle'])
+        self.assertIn('Buildbot commands', self.sent[0])
 
     @defer.inlineCallbacks
     def test_command_hello(self):
-        yield self.do_test_command('hello', exp_usage=False)
-        self.assertEqual(self.sent, ['yes?'])
         yield self.do_test_command('hello', exp_usage=False)
         self.assertIn(self.sent[0], words.GREETINGS)
 
@@ -358,32 +369,48 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
 
     @defer.inlineCallbacks
     def test_command_list_builders(self):
-        yield self.do_test_command('list', args='builders')
+        yield self.do_test_command('list', args='all builders')
         self.assertEqual(len(self.sent), 1)
         for builder in self.BUILDER_NAMES:
             self.assertIn('%s [offline]' % builder, self.sent[0])
 
-    def setup_multi_builders(self):
-        # Make first builder configured, but not connected
-        # Make second builder configured and connected
+    @defer.inlineCallbacks
+    def test_command_list_workers(self):
+        workers = ['worker1', 'worker2']
+        for worker in workers:
+            self.master.db.workers.db.insertTestData([
+                fakedb.Worker(name=worker)
+            ])
+        yield self.do_test_command('list', args='all workers')
+        self.assertEqual(len(self.sent), 1)
+        for worker in workers:
+            self.assertIn('%s [offline]' % worker, self.sent[0])
+
+    @defer.inlineCallbacks
+    def test_command_list_workers_online(self):
+        self.setup_multi_builders()
+        # Also set the connectedness:
         self.master.db.insertTestData([
-            fakedb.Worker(id=1, name='linux', info={}),  # connected one
-            fakedb.Worker(id=2, name='linux', info={}),  # disconnected one
-            fakedb.BuilderMaster(
-                id=4012, masterid=13, builderid=self.BUILDER_IDS[0]),
-            fakedb.BuilderMaster(
-                id=4013, masterid=13, builderid=self.BUILDER_IDS[1]),
-            fakedb.ConfiguredWorker(id=14013,
-                                    workerid=2, buildermasterid=4012),
-            fakedb.ConfiguredWorker(id=14013,
-                                    workerid=1, buildermasterid=4013),
+            fakedb.ConnectedWorker(id=113, masterid=13, workerid=1)
         ])
+        yield self.do_test_command('list', args='all workers')
+        self.assertEqual(len(self.sent), 1)
+        self.assertNotIn('linux1 [disconnected]', self.sent[0])
+        self.assertIn('linux2 [disconnected]', self.sent[0])
+
+    @defer.inlineCallbacks
+    def test_command_list_changes(self):
+        self.master.db.workers.db.insertTestData([
+            fakedb.Change()
+        ])
+        yield self.do_test_command('list', args='2 changes')
+        self.assertEqual(len(self.sent), 1)
 
     @defer.inlineCallbacks
     def test_command_list_builders_not_connected(self):
         self.setup_multi_builders()
 
-        yield self.do_test_command('list', args='builders')
+        yield self.do_test_command('list', args='all builders')
         self.assertEqual(len(self.sent), 1)
         self.assertIn('%s [offline]' % self.BUILDER_NAMES[0], self.sent[0])
         self.assertIn('%s [offline]' % self.BUILDER_NAMES[1], self.sent[0])
@@ -391,13 +418,12 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     @defer.inlineCallbacks
     def test_command_list_builders_connected(self):
         self.setup_multi_builders()
-
         # Also set the connectedness:
         self.master.db.insertTestData([
             fakedb.ConnectedWorker(id=113, masterid=13, workerid=1)
         ])
 
-        yield self.do_test_command('list', args='builders')
+        yield self.do_test_command('list', args='all builders')
         self.assertEqual(len(self.sent), 1)
         self.assertIn('%s [offline]' % self.BUILDER_NAMES[0], self.sent[0])
         self.assertNotIn('%s [offline]' % self.BUILDER_NAMES[1], self.sent[0])
@@ -406,6 +432,21 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     def test_command_status(self):
         yield self.do_test_command('status')
 
+    @defer.inlineCallbacks
+    def test_command_status_online(self):
+        # we are online and we have some finished builds
+        self.setup_multi_builders()
+        self.master.db.insertTestData([
+            fakedb.ConfiguredWorker(id=14012,
+                                    workerid=1, buildermasterid=4013),
+            fakedb.ConnectedWorker(id=114, masterid=13, workerid=1)
+        ])
+        self.setupSomeBuilds()
+        self.master.db.builds.finishBuild(buildid=13, results=FAILURE)
+        self.master.db.builds.finishBuild(buildid=15, results=SUCCESS)
+        self.master.db.builds.finishBuild(buildid=16, results=FAILURE)
+        yield self.do_test_command('status')
+
     @defer.inlineCallbacks
     def test_command_status_all(self):
         yield self.do_test_command('status', args='all')
@@ -413,16 +454,16 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     @defer.inlineCallbacks
     def test_command_status_builder0_offline(self):
         yield self.do_test_command('status', args=self.BUILDER_NAMES[0])
-        self.assertEqual(self.sent, ['%s: offline' % self.BUILDER_NAMES[0]])
+        self.assertEqual(self.sent, ['`%s`: offline' % self.BUILDER_NAMES[0]])
 
     @defer.inlineCallbacks
     def test_command_status_builder0_running(self):
         self.setupSomeBuilds()
         yield self.do_test_command('status', args=self.BUILDER_NAMES[0])
         self.assertEqual(len(self.sent), 1)
-        self.assertIn('builder1: running', self.sent[0])
-        self.assertIn(' 3 (no current step)', self.sent[0])
-        self.assertIn(' 6 (no current step)', self.sent[0])
+        self.assertIn('`builder1`: running', self.sent[0])
+        self.assertRegex(self.sent[0], r' build \[#3\].* \(no current step\)')
+        self.assertRegex(self.sent[0], r' build \[#6\].* \(no current step\)')
 
     @defer.inlineCallbacks
     def test_command_status_bogus(self):
@@ -430,18 +471,32 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
 
     def sub_seconds(self, strings):
         # sometimes timing works out wrong, so just call it "n seconds"
-        return [re.sub(r'\d seconds', 'N seconds', s) for s in strings]
+        return [re.sub(r'\d seconds|a moment', 'N seconds', s) for s in strings]
 
     @defer.inlineCallbacks
     def test_command_last(self):
         self.setupSomeBuilds()
+        self.setup_multi_builders()
+        # Also set the connectedness:
+        self.master.db.insertTestData([
+            fakedb.ConnectedWorker(id=113, masterid=13, workerid=2)
+        ])
         yield self.do_test_command('last')
-        self.assertEqual(len(self.sent), 2)
+        self.assertEqual(len(self.sent), 1)
         sent = self.sub_seconds(self.sent)
         self.assertIn(
-            'last build [builder1]: last build N seconds ago: test', sent)
+            '`builder1`: last build completed successfully (N seconds ago)', sent)
+
+    @defer.inlineCallbacks
+    def test_command_last_all(self):
+        self.setupSomeBuilds()
+        yield self.do_test_command('last', args='all')
+        self.assertEqual(len(self.sent), 1)
+        sent = self.sub_seconds(self.sent)
         self.assertIn(
-            'last build [builder2]: (no builds run since last restart)', sent)
+            '`builder1`: last build completed successfully (N seconds ago)', sent[0])
+        self.assertIn(
+            '`builder2`: no builds run since last restart', sent[0])
 
     @defer.inlineCallbacks
     def test_command_last_builder_bogus(self):
@@ -454,7 +509,7 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         self.assertEqual(len(self.sent), 1)
         sent = self.sub_seconds(self.sent)
         self.assertIn(
-            'last build [builder1]: last build N seconds ago: test', sent)
+            '`builder1`: last build completed successfully (N seconds ago)', sent)
 
     @defer.inlineCallbacks
     def test_command_last_builder1(self):
@@ -462,7 +517,7 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         yield self.do_test_command('last', args=self.BUILDER_NAMES[1])
         self.assertEqual(len(self.sent), 1)
         self.assertIn(
-            'last build [builder2]: (no builds run since last restart)', self.sent)
+            '`builder2`: no builds run since last restart', self.sent)
 
     @defer.inlineCallbacks
     def test_command_watch(self):
@@ -472,25 +527,7 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     def test_command_watch_builder0_no_builds(self):
         yield self.do_test_command('watch', args=self.BUILDER_NAMES[0])
         self.assertEqual(len(self.sent), 1)
-        self.assertIn('there are no builds', self.sent[0])
-
-    def setupSomeBuilds(self):
-        self.master.db.insertTestData([
-            # Three builds on builder#0, One build on builder#1
-            fakedb.Build(id=13, masterid=88, workerid=13,
-                         builderid=self.BUILDER_IDS[0],
-                         buildrequestid=82, number=3),
-            fakedb.Build(id=14, masterid=88, workerid=13,
-                         builderid=self.BUILDER_IDS[0],
-                         buildrequestid=83, number=4),
-            fakedb.Build(id=15, masterid=88, workerid=13,
-                         builderid=self.BUILDER_IDS[1],
-                         buildrequestid=84, number=5),
-            fakedb.Build(id=16, masterid=88, workerid=13,
-                         builderid=self.BUILDER_IDS[0],
-                         buildrequestid=85, number=6),
-        ])
-        self.master.db.builds.finishBuild(buildid=14, results='results')
+        self.assertIn('There are no currently running builds.', self.sent[0])
 
     @defer.inlineCallbacks
     def test_command_watch_builder0(self):
@@ -498,9 +535,11 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         yield self.do_test_command('watch', args=self.BUILDER_NAMES[0])
         self.assertEqual(len(self.sent), 2)
         self.assertIn(
-            'watching build builder1 #3 until it finishes..', self.sent)
+            'Watching build [#3](http://localhost:8080/#builders/23/builds/3) of `builder1` until it finishes...',
+            self.sent)
         self.assertIn(
-            'watching build builder1 #6 until it finishes..', self.sent)
+            'Watching build [#6](http://localhost:8080/#builders/23/builds/6) of `builder1` until it finishes...',
+            self.sent)
 
     @defer.inlineCallbacks
     def test_command_watch_builder0_get_notifications(self):
@@ -512,8 +551,8 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         yield self.sendBuildFinishedMessage(16)
         self.assertEqual(len(self.sent), 1)
         self.assertIn(
-                "Build builder1 #6 is complete: Success [] - "
-                "http://localhost:8080/#builders/23/builds/6", self.sent)
+            "Build [#6](http://localhost:8080/#builders/23/builds/6) of `builder1` completed successfully.",
+            self.sent)
 
     @defer.inlineCallbacks
     def test_command_watch_builder1(self):
@@ -521,9 +560,11 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         yield self.do_test_command('watch', args=self.BUILDER_NAMES[0])
         self.assertEqual(len(self.sent), 2)
         self.assertIn(
-            'watching build builder1 #3 until it finishes..', self.sent)
+            'Watching build [#3](http://localhost:8080/#builders/23/builds/3) of `builder1` until it finishes...',
+            self.sent)
         self.assertIn(
-            'watching build builder1 #6 until it finishes..', self.sent)
+            'Watching build [#6](http://localhost:8080/#builders/23/builds/6) of `builder1` until it finishes...',
+            self.sent)
 
     @defer.inlineCallbacks
     def sendBuildFinishedMessage(self, buildid, results=0):
@@ -565,8 +606,8 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         self.setupSomeBuilds()
         yield self.do_test_command('stop', args="build %s 'i have a reason'" % self.BUILDER_NAMES[0])
         self.assertEqual(len(self.sent), 2)
-        self.assertIn('build 3 interrupted', self.sent)
-        self.assertIn('build 6 interrupted', self.sent)
+        self.assertRegex(self.sent[0], r'Build \[#[36]\].* of `builder1` interrupted')
+        self.assertRegex(self.sent[1], r'Build \[#[63]\].* of `builder1` interrupted')
 
     @defer.inlineCallbacks
     def test_command_force_no_args(self):
@@ -584,54 +625,12 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
     #   - bad args
     #   - arg validation failure (self.master.config.validation)
 
-    # TODO: the below fails due to the assertion (see "rewrite to not use the status hierarchy")
-    # @defer.inlineCallbacks
-    # def test_command_force(self):
-    #     yield self.do_test_command('force',
-    #             args='build --branch BRANCH1 --revision REV1 --props=PROP1=VALUE1 %s REASON'
-    #               % self.BUILDER_NAMES[0])
-
-    def test_send(self):
-        events = []
-
-        def groupChat(dest, msg):
-            events.append((dest, msg))
-        self.contact.bot.groupChat = groupChat
-
-        self.contact.send("unmuted")
-        self.contact.send("unmuted, unicode \N{SNOWMAN}")
-        self.contact.muted = True
-        self.contact.send("muted")
-
-        self.assertEqual(events, [
-            ('#buildbot', 'unmuted'),
-            ('#buildbot', 'unmuted, unicode \u2603'),
-        ])
-
-    def test_act(self):
-        events = []
-
-        def groupDescribe(dest, msg):
-            events.append((dest, msg))
-        self.contact.bot.groupDescribe = groupDescribe
-
-        self.contact.act("unmuted")
-        self.contact.act("unmuted, unicode \N{SNOWMAN}")
-        self.contact.muted = True
-        self.contact.act("muted")
-
-        self.assertEqual(events, [
-            ('#buildbot', 'unmuted'),
-            ('#buildbot', 'unmuted, unicode \u2603'),
-        ])
-
     @defer.inlineCallbacks
-    def test_handleMessage_silly(self):
-        silly_prompt = list(self.contact.silly)[0]
-        self.contact.doSilly = mock.Mock()
-        yield self.contact.handleMessage(silly_prompt)
-
-        self.contact.doSilly.assert_called_with(silly_prompt)
+    def test_command_force(self):
+        yield self.do_test_command(
+            'force',
+            args='build --branch BRANCH1 --revision REV1 --props=PROP1=VALUE1 {} REASON'
+                 .format(self.BUILDER_NAMES[0]))
 
     @defer.inlineCallbacks
     def test_handleMessage_short_command(self):
@@ -678,21 +677,6 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
 
         self.assertEqual(self.sent, ["oh noes"])
 
-    def test_handleAction_ignored(self):
-        self.patch_act()
-        self.contact.handleAction('waves hi')
-        self.assertEqual(self.actions, [])
-
-    def test_handleAction_kick(self):
-        self.patch_act()
-        self.contact.handleAction('kicks nick')
-        self.assertEqual(self.actions, ['kicks back'])
-
-    def test_handleAction_stupid(self):
-        self.patch_act()
-        self.contact.handleAction('stupids nick')
-        self.assertEqual(self.actions, ['stupids me too'])
-
     @defer.inlineCallbacks
     def test_unclosed_quote(self):
         yield self.do_test_command('list', args='args\'', exp_UsageError=True)
@@ -711,26 +695,72 @@ class TestContactChannel(TestReactorMixin, unittest.TestCase):
         build = yield self.master.db.builds.getBuild(13)
 
         self.bot.tags = None
-        self.contact.notify_for = lambda _: True
+        self.contact.channel.notify_for = lambda _: True
         self.contact.useRevisions = False
 
-        self.contact.buildStarted(build)
+        self.contact.channel.buildStarted(build)
         self.assertEqual(
             self.sent.pop(),
-            "build #3 of builder1 started")
+            "Build [#3](http://localhost:8080/#builders/23/builds/3) of `builder1` started.")
 
+    def test_getCommandMethod_authz_default(self):
+        self.bot.authz = words.StatusBot.expand_authz(None)
+        meth = self.contact.getCommandMethod('shutdown')
+        self.assertEqual(meth, self.contact.access_denied)
 
-class FakeContact:
+    authz1 = {
+        'force': ['me'],
+        'shutdown': ['notme', 'someone'],
+        ('dance', 'notify'): True,
+        '': False}
 
-    def __init__(self, bot, user=None, channel=None):
-        self.bot = bot
-        self.user = user
-        self.channel = channel
-        self.messages = []
-        self.actions = []
+    def test_getCommandMethod_explicit_allow(self):
+        self.bot.authz = words.StatusBot.expand_authz(self.authz1)
+        meth = self.contact.getCommandMethod('force')
+        self.assertNotEqual(meth, self.contact.access_denied)
 
-    def handleMessage(self, message):
-        self.messages.append(message)
+    def test_getCommandMethod_explicit_disallow(self):
+        self.bot.authz = words.StatusBot.expand_authz(self.authz1)
+        meth = self.contact.getCommandMethod('shutdown')
+        self.assertEqual(meth, self.contact.access_denied)
 
-    def handleAction(self, data):
-        self.actions.append(data)
+    def test_getCommandMethod_explicit_multi(self):
+        self.bot.authz = words.StatusBot.expand_authz(self.authz1)
+        self.assertIn('DANCE', self.bot.authz)
+        meth = self.contact.getCommandMethod('dance')
+        self.assertNotEqual(meth, self.contact.access_denied)
+
+    def test_getCommandMethod_explicit_default(self):
+        self.bot.authz = words.StatusBot.expand_authz(self.authz1)
+        meth = self.contact.getCommandMethod('help')
+        self.assertEqual(meth, self.contact.access_denied)
+
+    authz2 = {
+        'shutdown': False,
+        '': False,
+        '*': True}
+
+    def test_getCommandMethod_exclamation(self):
+        self.bot.authz = words.StatusBot.expand_authz(self.authz2)
+        meth = self.contact.getCommandMethod('help')
+        self.assertNotEqual(meth, self.contact.access_denied)
+
+    def test_getCommandMethod_exclamation_override(self):
+        self.bot.authz = words.StatusBot.expand_authz(self.authz2)
+        meth = self.contact.getCommandMethod('shutdown')
+        self.assertEqual(meth, self.contact.access_denied)
+
+    def test_access_denied(self):
+        self.patch_send()
+        self.contact.access_denied()
+        self.assertIn("not pass", self.sent[0])
+
+    @defer.inlineCallbacks
+    def test_bot_loadState(self):
+        boid = yield self.bot._get_object_id()
+        self.master.db.insertTestData([
+            fakedb.ObjectState(objectid=boid, name='notify_events',
+                               value_json='[["#channel1", ["warnings"]]]'),
+        ])
+        yield self.bot.loadState()
+        self.assertEqual(self.bot.channels['#channel1'].notify_events, {'warnings'})
