@@ -14,6 +14,7 @@
 # Copyright Buildbot Team Members
 
 
+from twisted.internet import defer
 from twisted.trial import unittest
 
 from buildbot import config
@@ -27,15 +28,16 @@ from buildbot.test.util.misc import TestReactorMixin
 from buildbot.worker import docker as dockerworker
 
 
-class TestDockerLatentWorker(unittest.SynchronousTestCase, TestReactorMixin):
+class TestDockerLatentWorker(unittest.TestCase, TestReactorMixin):
 
+    @defer.inlineCallbacks
     def setupWorker(self, *args, **kwargs):
         self.patch(dockerworker, 'docker', docker)
         worker = dockerworker.DockerLatentWorker(*args, **kwargs)
         master = fakemaster.make_master(self, wantData=True)
         fakemaster.master = master
         worker.setServiceParent(master)
-        self.successResultOf(master.startService())
+        yield master.startService()
         self.addCleanup(master.stopService)
         return worker
 
@@ -46,68 +48,77 @@ class TestDockerLatentWorker(unittest.SynchronousTestCase, TestReactorMixin):
             image='busybox:latest', builder='docker_worker', distro='wheezy')
         self.patch(dockerworker, 'client', docker)
 
+    @defer.inlineCallbacks
     def test_constructor_nodocker(self):
         self.patch(dockerworker, 'client', None)
         with self.assertRaises(config.ConfigErrors):
-            self.setupWorker('bot', 'pass', 'unix://tmp.sock', 'debian:wheezy',
-                             [])
+            yield self.setupWorker('bot', 'pass', 'unix://tmp.sock', 'debian:wheezy', [])
 
+    @defer.inlineCallbacks
     def test_constructor_noimage_nodockerfile(self):
         with self.assertRaises(config.ConfigErrors):
-            self.setupWorker('bot', 'pass', 'http://localhost:2375')
+            yield self.setupWorker('bot', 'pass', 'http://localhost:2375')
 
+    @defer.inlineCallbacks
     def test_constructor_noimage_dockerfile(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'http://localhost:2375', dockerfile="FROM ubuntu")
         self.assertEqual(bs.dockerfile, "FROM ubuntu")
         self.assertEqual(bs.image, None)
 
+    @defer.inlineCallbacks
     def test_constructor_image_nodockerfile(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'http://localhost:2375', image="myworker")
         self.assertEqual(bs.dockerfile, None)
         self.assertEqual(bs.image, 'myworker')
 
+    @defer.inlineCallbacks
     def test_constructor_minimal(self):
         # Minimal set of parameters
-        bs = self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker')
+        bs = yield self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker')
         self.assertEqual(bs.workername, 'bot')
         self.assertEqual(bs.password, 'pass')
         self.assertEqual(bs.client_args, {'base_url': 'tcp://1234:2375'})
         self.assertEqual(bs.image, 'worker')
         self.assertEqual(bs.command, [])
 
+    @defer.inlineCallbacks
     def test_contruction_minimal_docker_py(self):
         docker.version = "1.10.6"
-        bs = self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker')
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        bs = yield self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker')
+        id, name = yield bs.start_instance(self.build)
         client = docker.APIClient.latest
         self.assertEqual(client.called_class_name, "Client")
         client = docker.Client.latest
         self.assertNotEqual(client.called_class_name, "APIClient")
 
+    @defer.inlineCallbacks
     def test_contruction_minimal_docker(self):
         docker.version = "2.0.0"
-        bs = self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker')
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        bs = yield self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker')
+        id, name = yield bs.start_instance(self.build)
         client = docker.Client.latest
         self.assertEqual(client.called_class_name, "APIClient")
         client = docker.APIClient.latest
         self.assertNotEqual(client.called_class_name, "Client")
 
+    @defer.inlineCallbacks
     def test_constructor_nopassword(self):
         # when no password, it is created automatically
-        bs = self.setupWorker('bot', None, 'tcp://1234:2375', 'worker')
+        bs = yield self.setupWorker('bot', None, 'tcp://1234:2375', 'worker')
         self.assertEqual(bs.workername, 'bot')
         self.assertEqual(len(bs.password), 20)
 
+    @defer.inlineCallbacks
     def test_constructor_all_docker_parameters(self):
         # Volumes have their own tests
-        bs = self.setupWorker('bot', 'pass', 'unix:///var/run/docker.sock', 'worker_img', ['/bin/sh'],
-                              dockerfile="FROM ubuntu", version='1.9', tls=True,
-                              hostconfig={'network_mode': 'fake', 'dns': ['1.1.1.1', '1.2.3.4']},
-                              custom_context=False, buildargs=None,
-                              encoding='gzip')
+        bs = yield self.setupWorker('bot', 'pass', 'unix:///var/run/docker.sock', 'worker_img',
+                                    ['/bin/sh'],
+                                    dockerfile="FROM ubuntu", version='1.9', tls=True,
+                                    hostconfig={'network_mode': 'fake', 'dns': ['1.1.1.1', '1.2.3.4']},
+                                    custom_context=False, buildargs=None,
+                                    encoding='gzip')
         self.assertEqual(bs.workername, 'bot')
         self.assertEqual(bs.password, 'pass')
         self.assertEqual(bs.image, 'worker_img')
@@ -122,165 +133,185 @@ class TestDockerLatentWorker(unittest.SynchronousTestCase, TestReactorMixin):
         self.assertEqual(bs.buildargs, None)
         self.assertEqual(bs.encoding, 'gzip')
 
+    @defer.inlineCallbacks
     def test_start_instance_volume_renderable(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'worker', ['bin/bash'],
             volumes=[Interpolate('/data:/worker/%(kw:builder)s/build',
                                  builder=Property('builder'))])
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         client = docker.Client.latest
         self.assertEqual(len(client.call_args_create_container), 1)
         self.assertEqual(client.call_args_create_container[0]['volumes'],
                          ['/worker/docker_worker/build'])
 
+    @defer.inlineCallbacks
     def test_volume_no_suffix(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'worker', ['bin/bash'], volumes=['/src/webapp:/opt/webapp'])
-        self.successResultOf(bs.start_instance(self.build))
+        yield bs.start_instance(self.build)
         client = docker.Client.latest
         self.assertEqual(len(client.call_args_create_container), 1)
         self.assertEqual(len(client.call_args_create_host_config), 1)
         self.assertEqual(client.call_args_create_container[0]['volumes'],
                          ['/opt/webapp'])
         self.assertEqual(client.call_args_create_host_config[0]['binds'],
-                         {'/src/webapp': {'bind': '/opt/webapp', 'ro': False}})
+                         ["/src/webapp:/opt/webapp"])
 
+    @defer.inlineCallbacks
     def test_volume_ro_rw(self):
-        bs = self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker', ['bin/bash'],
-                              volumes=['/src/webapp:/opt/webapp:ro',
-                                       '~:/backup:rw'])
-        self.successResultOf(bs.start_instance(self.build))
+        bs = yield self.setupWorker('bot', 'pass', 'tcp://1234:2375', 'worker', ['bin/bash'],
+                                    volumes=['/src/webapp:/opt/webapp:ro',
+                                             '~:/backup:rw'])
+        yield bs.start_instance(self.build)
         client = docker.Client.latest
         self.assertEqual(len(client.call_args_create_container), 1)
         self.assertEqual(len(client.call_args_create_host_config), 1)
         self.assertEqual(client.call_args_create_container[0]['volumes'],
                          ['/opt/webapp', '/backup'])
         self.assertEqual(client.call_args_create_host_config[0]['binds'],
-                         {'/src/webapp': {'bind': '/opt/webapp', 'ro': True},
-                          '~': {'bind': '/backup', 'ro': False}})
+                         ['/src/webapp:/opt/webapp:ro', '~:/backup:rw'])
 
+    @defer.inlineCallbacks
     def test_volume_bad_format(self):
         with self.assertRaises(config.ConfigErrors):
-            self.setupWorker('bot', 'pass', 'http://localhost:2375',
-                             image="worker",
-                             volumes=['abcd=efgh'])
+            yield self.setupWorker('bot', 'pass', 'http://localhost:2375',
+                                   image="worker",
+                                   volumes=['abcd=efgh'])
 
+    @defer.inlineCallbacks
     def test_volume_bad_format_renderable(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'http://localhost:2375', image="worker",
             volumes=[Interpolate('/data==/worker/%(kw:builder)s/build',
                                  builder=Property('builder'))])
-        f = self.failureResultOf(bs.start_instance(self.build))
-        f.check(config.ConfigErrors)
+        with self.assertRaises(config.ConfigErrors):
+            yield bs.start_instance(self.build)
 
+    @defer.inlineCallbacks
     def test_start_instance_image_no_version(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'busybox', ['bin/bash'])
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'busybox')
 
+    @defer.inlineCallbacks
     def test_start_instance_image_right_version(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'busybox:latest', ['bin/bash'])
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'busybox:latest')
 
+    @defer.inlineCallbacks
     def test_start_instance_image_wrong_version(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'busybox:123', ['bin/bash'])
-        f = self.failureResultOf(bs.start_instance(self.build))
-        f.check(interfaces.LatentWorkerFailedToSubstantiate)
+        with self.assertRaises(interfaces.LatentWorkerCannotSubstantiate):
+            yield bs.start_instance(self.build)
 
+    @defer.inlineCallbacks
     def test_start_instance_image_renderable(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', Property('image'), ['bin/bash'])
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'busybox:latest')
 
+    @defer.inlineCallbacks
     def test_start_instance_noimage_nodockerfile(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'customworker', ['bin/bash'])
-        f = self.failureResultOf(bs.start_instance(self.build))
-        f.check(interfaces.LatentWorkerFailedToSubstantiate)
+        with self.assertRaises(interfaces.LatentWorkerCannotSubstantiate):
+            yield bs.start_instance(self.build)
 
+    @defer.inlineCallbacks
     def test_start_instance_image_and_dockefile(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'customworker', dockerfile='BUG')
-        f = self.failureResultOf(bs.start_instance(self.build))
-        f.check(interfaces.LatentWorkerFailedToSubstantiate)
+        with self.assertRaises(interfaces.LatentWorkerCannotSubstantiate):
+            yield bs.start_instance(self.build)
 
+    @defer.inlineCallbacks
     def test_start_instance_noimage_gooddockerfile(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'customworker', dockerfile='FROM debian:wheezy')
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'customworker')
 
+    @defer.inlineCallbacks
     def test_start_instance_noimage_pull(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'alpine:latest', autopull=True)
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'alpine:latest')
 
+    @defer.inlineCallbacks
     def test_start_instance_image_pull(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'tester:latest', autopull=True)
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'tester:latest')
         client = docker.Client.latest
         self.assertEqual(client._pullCount, 0)
 
+    @defer.inlineCallbacks
     def test_start_instance_image_alwayspull(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'tester:latest', autopull=True, alwaysPull=True)
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'tester:latest')
         client = docker.Client.latest
         self.assertEqual(client._pullCount, 1)
 
+    @defer.inlineCallbacks
     def test_start_instance_image_noauto_alwayspull(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'tester:latest', autopull=False, alwaysPull=True)
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'tester:latest')
         client = docker.Client.latest
         self.assertEqual(client._pullCount, 0)
 
+    @defer.inlineCallbacks
     def test_start_instance_noimage_renderabledockerfile(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'customworker',
             dockerfile=Interpolate('FROM debian:%(kw:distro)s',
                                    distro=Property('distro')))
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'customworker')
 
+    @defer.inlineCallbacks
     def test_start_instance_custom_context_and_buildargs(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'tester:latest',
             dockerfile=Interpolate('FROM debian:latest'), custom_context=True,
             buildargs={'sample_arg1': 'test_val1'})
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'tester:latest')
 
+    @defer.inlineCallbacks
     def test_start_instance_custom_context_no_buildargs(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'tester:latest',
             dockerfile=Interpolate('FROM debian:latest'),
             custom_context=True)
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'tester:latest')
 
+    @defer.inlineCallbacks
     def test_start_instance_buildargs_no_custom_context(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'bot', 'pass', 'tcp://1234:2375', 'tester:latest',
             dockerfile=Interpolate('FROM debian:latest'),
             buildargs={'sample_arg1': 'test_val1'})
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'tester:latest')
 
+    @defer.inlineCallbacks
     def test_start_worker_but_already_created_with_same_name(self):
-        bs = self.setupWorker(
+        bs = yield self.setupWorker(
             'existing', 'pass', 'tcp://1234:2375', 'busybox:latest', ['bin/bash'])
-        id, name = self.successResultOf(bs.start_instance(self.build))
+        id, name = yield bs.start_instance(self.build)
         self.assertEqual(name, 'busybox:latest')
 
 
