@@ -26,7 +26,9 @@ import requests
 from twisted.internet import defer
 from twisted.internet import threads
 
+import buildbot
 from buildbot import config
+from buildbot.process.properties import Properties
 from buildbot.util import bytes2unicode
 from buildbot.util.logger import Logger
 from buildbot.www import auth
@@ -49,27 +51,24 @@ class OAuth2LoginResource(auth.LoginResource):
     @defer.inlineCallbacks
     def renderLogin(self, request):
         code = request.args.get(b"code", [b""])[0]
-        token = request.args.get(b"token", [b""])[0]
-        if not token and not code:
+        if not code:
             url = request.args.get(b"redirect", [None])[0]
             url = yield self.auth.getLoginURL(url)
             raise resource.Redirect(url)
-        else:
-            if not token:
-                details = yield self.auth.verifyCode(code)
-            else:
-                details = yield self.auth.acceptToken(token)
-            if self.auth.userInfoProvider is not None:
-                infos = yield self.auth.userInfoProvider.getUserInfo(details['username'])
-                details.update(infos)
-            session = request.getSession()
-            session.user_info = details
-            session.updateSession(request)
-            state = request.args.get(b"state", [b""])[0]
-            if state:
-                for redirect in parse_qs(state).get('redirect', []):
-                    raise resource.Redirect(self.auth.homeUri + "#" + redirect)
-            raise resource.Redirect(self.auth.homeUri)
+
+        details = yield self.auth.verifyCode(code)
+
+        if self.auth.userInfoProvider is not None:
+            infos = yield self.auth.userInfoProvider.getUserInfo(details['username'])
+            details.update(infos)
+        session = request.getSession()
+        session.user_info = details
+        session.updateSession(request)
+        state = request.args.get(b"state", [b""])[0]
+        if state:
+            for redirect in parse_qs(state).get('redirect', []):
+                raise resource.Redirect(self.auth.homeUri + "#" + redirect)
+        raise resource.Redirect(self.auth.homeUri)
 
 
 class OAuth2Auth(auth.AuthBase):
@@ -106,17 +105,21 @@ class OAuth2Auth(auth.AuthBase):
     def getLoginResource(self):
         return OAuth2LoginResource(self.master, self)
 
+    @defer.inlineCallbacks
     def getLoginURL(self, redirect_url):
         """
         Returns the url to redirect the user to for user consent
         """
+        p = Properties()
+        p.master = self.master
+        clientId = yield p.render(self.clientId)
         oauth_params = {'redirect_uri': self.loginUri,
-                        'client_id': self.clientId, 'response_type': 'code'}
+                        'client_id': clientId, 'response_type': 'code'}
         if redirect_url is not None:
             oauth_params['state'] = urlencode(dict(redirect=redirect_url))
         oauth_params.update(self.authUriAdditionalParams)
         sorted_oauth_params = sorted(oauth_params.items(), key=lambda val: val[0])
-        return defer.succeed("%s?%s" % (self.authUri, urlencode(sorted_oauth_params)))
+        return "%s?%s" % (self.authUri, urlencode(sorted_oauth_params))
 
     def createSessionFromToken(self, token):
         s = requests.Session()
@@ -128,29 +131,22 @@ class OAuth2Auth(auth.AuthBase):
         ret = session.get(self.resourceEndpoint + path)
         return ret.json()
 
-    # If the user wants to authenticate directly with an access token they
-    # already have, go ahead and just directly accept an access_token from them.
-    def acceptToken(self, token):
-        def thd():
-            session = self.createSessionFromToken({'access_token': token})
-            return self.getUserInfoFromOAuthClient(session)
-        return threads.deferToThread(thd)
-
     # based on https://github.com/maraujop/requests-oauth
     # from Miguel Araujo, augmented to support header based clientSecret
     # passing
+    @defer.inlineCallbacks
     def verifyCode(self, code):
         # everything in deferToThread is not counted with trial  --coverage :-(
-        def thd():
+        def thd(client_id, client_secret):
             url = self.tokenUri
             data = {'redirect_uri': self.loginUri, 'code': code,
                     'grant_type': self.grantType}
             auth = None
             if self.getTokenUseAuthHeaders:
-                auth = (self.clientId, self.clientSecret)
+                auth = (client_id, client_secret)
             else:
                 data.update(
-                    {'client_id': self.clientId, 'client_secret': self.clientSecret})
+                    {'client_id': client_id, 'client_secret': client_secret})
             data.update(self.tokenUriAdditionalParams)
             response = requests.post(
                 url, data=data, auth=auth, verify=self.sslVerify)
@@ -167,7 +163,12 @@ class OAuth2Auth(auth.AuthBase):
 
             session = self.createSessionFromToken(content)
             return self.getUserInfoFromOAuthClient(session)
-        return threads.deferToThread(thd)
+        p = Properties()
+        p.master = self.master
+        client_id = yield p.render(self.clientId)
+        client_secret = yield p.render(self.clientSecret)
+        result = yield threads.deferToThread(thd, client_id, client_secret)
+        return result
 
     def getUserInfoFromOAuthClient(self, c):
         return {}
@@ -284,6 +285,15 @@ class GitHubAuth(OAuth2Auth):
                     username=user['login'],
                     groups=[org['login'] for org in orgs])
 
+    def createSessionFromToken(self, token):
+        s = requests.Session()
+        s.headers = {
+            'Authorization': 'token ' + token['access_token'],
+            'User-Agent': 'buildbot/%s' % buildbot.version,
+        }
+        s.verify = self.sslVerify
+        return s
+
     def getUserInfoFromOAuthClient_v4(self, c):
         graphql_query = textwrap.dedent('''
             query {
