@@ -13,8 +13,6 @@
 #
 # Copyright  Team Members
 
-from future.utils import string_types
-
 import json
 
 from autobahn.twisted.resource import WebSocketResource
@@ -71,7 +69,7 @@ class WsProtocol(WebSocketServerProtocol):
         return tuple([str(p) if p != "*" else None for p in path])
 
     def isPath(self, path):
-        if not isinstance(path, string_types):
+        if not isinstance(path, str):
             return False
         return True
 
